-- 
-- Create tables for validation dictionary
-- 
-- This script is used when creating validation dictionary tables
-- from main NMR-STAR dictionary tables. Main dictionary does not
-- include enumerations for mandatory codes and data types.
--
DROP VIEW DICT;
DROP TABLE INFO;
DROP TABLE STARCH;
DROP TABLE DATUMTYPES;
DROP TABLE TAGDEPS;
DROP TABLE VALENUMS;
DROP TABLE TAGRELS;
DROP TABLE TAGS;
DROP TABLE SFCATS;
DROP TABLE SFMANENUM;
DROP TABLE TAGMANENUM;
DROP TABLE VALTYPENUM;
DROP TABLE DDLTYPES;
COMMIT;

--
--  utility tables
--
CREATE TABLE INFO (
  DMODE INTEGER NOT NULL,
  VERSION VARCHAR(15) NOT NULL,
  DICTFLAG CHAR(1) NOT NULL
);

GRANT ALL ON INFO TO PUBLIC;

-- 
-- enumeration of SF mandatory status codes
-- 
CREATE TABLE SFMANENUM (
 MANDATORY CHAR(1) PRIMARY KEY,
 DESCRIPTION VARCHAR(80) 
);

GRANT ALL ON SFMANENUM TO PUBLIC;

--
-- Tag mandatory status codes
--
CREATE TABLE TAGMANENUM (
 MANDATORY char(1) PRIMARY KEY,
 DESCRIPTION varchar(80) );

GRANT ALL ON TAGMANENUM TO PUBLIC;

-- 
-- Data types
-- 
CREATE TABLE VALTYPENUM (
 VALTYPE VARCHAR(9) PRIMARY KEY,
 DESCRIPTION VARCHAR(80) );

GRANT ALL ON VALTYPENUM TO PUBLIC;

--
-- SF categories 
--
CREATE TABLE SFCATS (
 ID INTEGER PRIMARY KEY,
 SFCAT varchar(80) NOT NULL,
 UNIQ char(1) NOT NULL,
 MANDATORY char(1) NOT NULL, 
 UNIQUE (SFCAT)
-- ,FOREIGN KEY (MANDATORY) REFERENCES SFMANENUM (MANDATORY)
);

GRANT ALL ON SFCATS TO PUBLIC;


--
-- Tags
--
CREATE TABLE TAGS (
 SEQ INTEGER PRIMARY KEY,
 SFCAT VARCHAR(80) NOT NULL,
 TAGNAME VARCHAR(127) NOT NULL,
 TAGCAT VARCHAR(60) NOT NULL,
 DBTABLE VARCHAR(127), -- NOT NULL,
 DBCOLUMN VARCHAR(127), -- NOT NULL,
 DBTYPE VARCHAR(127), -- NOT NULL,
 DBNOTNULL CHAR(1),
 DBPK CHAR(1),
 DBFKTABLE VARCHAR(127),
 DBFKCOLUMN VARCHAR(127),
 DBFKGROUP VARCHAR(5),
 VALTYPE VARCHAR(9) NOT NULL,
 VALSIZE INTEGER,
 MANDATORY CHAR(1) NOT NULL,
 TAGDEPFLAG CHAR(1),
 ENUMCLOSEDFLAG CHAR(1),
 ROWIDXFLAG CHAR(1),
 LOCALIDFLAG CHAR(1),
 SFIDFLAG CHAR(1),
 ENTRYIDFLAG CHAR(1),
 SFLABELFLAG CHAR(1),
 SFCATFLAG CHAR(1),
 SFLINKFLAG CHAR(1),
 LOOPFLAG CHAR(1),
 LOOPMANDATORY CHAR(1),
 DATUMCOUNT CHAR(1),
 METADATA CHAR(1),
 DELETEFLAG CHAR(1),
 ADITDEFAULT VARCHAR(80),
 ADITAUTO CHAR(1)
-- , FOREIGN KEY (SFCAT) REFERENCES SFCATS (SFCAT),
-- FOREIGN KEY (MANDATORY) REFERENCES TAGMANENUM (MANDATORY),
-- FOREIGN KEY (VALTYPE) REFERENCES VALTYPENUM (VALTYPE)
);

GRANT ALL ON TAGS TO PUBLIC;

-- 
-- Value enumerations
-- 
CREATE TABLE VALENUMS (
 SEQ INTEGER NOT NULL,
 VAL VARCHAR(127) NOT NULL
-- , FOREIGN KEY (SEQ) REFERENCES TAGS (SEQ) -- TAGCAT not unique in tags
);

CREATE UNIQUE INDEX VALENUMIDX ON VALENUMS (SEQ, VAL);

GRANT ALL ON VALENUMS TO PUBLIC;

-- GRANT ALL ON SFDEPS TO PUBLIC;

--
-- Dependent values
--
CREATE TABLE TAGDEPS (
 CTLSEQ INTEGER NOT NULL,
 CTLVALUE VARCHAR(255),
 MANDATORY CHAR(1) NOT NULL,
 SEQ INTEGER NOT NULL
-- ,FOREIGN KEY (MANDATORY) REFERENCES TAGMANENUM (MANDATORY),
-- FOREIGN KEY (CTLSEQ) REFERENCES TAGS (SEQ),
-- FOREIGN KEY (SEQ) REFERENCES TAGS (SEQ)
);

GRANT ALL ON TAGDEPS TO PUBLIC;

-- 
-- parent-child relationships
-- 
CREATE TABLE TAGRELS (
 PRNTSEQ INTEGER NOT NULL,
 CHLDSEQ INTEGER NOT NULL
-- , FOREIGN KEY (PRNTSEQ) REFERENCES TAGS (SEQ),
-- FOREIGN KEY (CHLDSEQ) REFERENCES TAGS (SEQ)
);

GRANT ALL ON TAGRELS TO PUBLIC;

--
-- Datum types
--
CREATE TABLE DATUMTYPES (
 TAGCAT VARCHAR(60) PRIMARY KEY,
 DATUMTYPE VARCHAR(127) NOT NULL,
 TABLEGEN CHAR(1) NOT NULL
);

GRANT ALL ON DATUMTYPES TO PUBLIC;

--
-- STARch
--
CREATE TABLE STARCH (
 TAGNAME VARCHAR(127) PRIMARY KEY,
 DISPLSEQ INTEGER NOT NULL,
 DISPLNAME VARCHAR(63) NOT NULL,
 SEQID CHAR(1),
 COMPIDXID CHAR(1),
 COMPID CHAR(1),
 ATOMID CHAR(1),
 ATOMTYPE CHAR(1),
 ISOTOPE CHAR(1),
 AMBICODE CHAR(1),
 GROUPID CHAR(1),
 VAL CHAR(1),
 MINVAL CHAR(1),
 MAXVAL CHAR(1),
 ERR CHAR(1),
 ROWIDX CHAR(1),
 AUTHOR CHAR(1),
 TABLEGEN CHAR(1)
-- ,FOREIGN KEY (TAGNAME) REFERENCES TAGS (TAGNAME)
);

GRANT ALL ON STARCH TO PUBLIC;

--
-- CIF data types
--
CREATE TABLE DDLTYPES(
 DDLTYPE VARCHAR( 24 ) PRIMARY KEY,
 REGEXP VARCHAR( 254 ),
 DESCRIPTION VARCHAR( 2047 )
);
